//= require <dSS/GetTextHelper>
//= require <dSS/config/Config>
Ext.namespace("dSS", "dSS.devices", "dSS.config");

// group
//  stop button will be shown depending on the group
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.addEvents('loading');

        this.dsid = config.dsid;
        this.previousBladeAngle = -1;

        this.maxValue = 0xff;
        var decPrecision = 0;

        if (config.matrix.shadeposition === true) {
            this.maxValue = 0xffff;
            decPrecision = 3;
        }

        this.matrix = config.matrix;

        this.valueLabel = new Ext.form.Label({
            x: 10,
            y: 16,
            text: dSS._("Value") + ':'
        });

        this.valueField = new Ext.form.NumberField({
            x: 120,
            y: 12,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: (decPrecision !== 0),
            decimalPrecision: decPrecision,
            forceDecimalPrecision: true,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateSlider.createDelegate(this)
            }
        });

        this.bladeAngleLabel = new Ext.form.Label({
            x: 10,
            y: 58,
            text: dSS._("Set blade angle") + ':'
        });

        this.bladeAngleField = new Ext.form.NumberField({
            x: 120,
            y: 54,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: false,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.handleBladeAngleChanged.createDelegate(this)
            }
        });

        this.bladeAngleUnit = new Ext.form.Label({
            x: 175,
            y: 58,
            text: '%'
        });

        this.valueSlider = new Ext.Slider({
            height: 150,
            minValue: 0,
            increment: (decPrecision === 0) ? 1 : 0.001,
            maxValue: 100,
            vertical: true,
            decimalPrecision: decPrecision,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleSliderChanged.createDelegate(this)
            }
        });

        this.stopButton = new Ext.Button({
            x: 120,
            y: 140,
            xtype: 'button',
            text: dSS._("Stop"),
            width: 50,
            handler: this.handleStop.createDelegate(this)
        });

        this.dirtyValue = false;
        this.dirtyAngle = false;

        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            height: 176,
            items:
            [
                this.valueLabel,
                this.valueField,
                {
                    x: 175,
                    y: 16,
                    xtype: 'label',
                    text: '%'
                },
                {
                    x: 190,
                    y: 12,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.valueSlider ]
                },
                {
                    xtype: 'button',
                    x: 222,
                    y: 12,
                    text: dSS._("Max"),
                    tooltip: dSS._("Set maximum value"),
                    width: 50,
                    handler: this.handleMax.createDelegate(this)
                },
                {
                    x: 222,
                    y: 54,
                    xtype: 'button',
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._("Increase value"),
                    width: 50,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    x: 222,
                    y: 98,
                    xtype: 'button',
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._("Decrease value"),
                    width: 50,
                    handler: this.handleDec.createDelegate(this)
                },
                this.stopButton,
                {
                    x: 222,
                    y: 140,
                    xtype: 'button',
                    text: dSS._("Min"),
                    tooltip: dSS._("Set minimum value"),
                    width: 50,
                    handler: this.handleMin.createDelegate(this)
                },
                this.bladeAngleLabel,
                this.bladeAngleField,
                this.bladeAngleUnit
            ]
        });

        this.setData(this.dsid, config.group, config.present, config.matrix);

        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
                this.killRequest();
                this.closed = true;
            },
            this);

        this.valueField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    var ov = this.valueSlider.getValue();
                    if (nv !== ov) {
                        this.updateSlider(field, nv, ov);
                    }
                }
            },
            this);

        this.bladeAngleField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    this.handleBladeAngleChanged(field, nv, this.previousBladeAngle);
                }
            },
            this);
    },
    percentToValue: function(percent) {
        var value = percent / 100;
        value = value * this.maxValue;
        if (value < 0) {
            value = 0;
        } else if (value > this.maxValue) {
            value = this.maxValue;
        }
        return Math.round(value);
    },
    valueToPercent: function(value) {
        if (value < 0) {
            value = 0;
        }  else if (value > this.maxValue) {
            value = this.maxValue;
        }
        return (value / this.maxValue * 100);
    },
    handleSliderChanged: function(slider, newVal, thumb) {
        this.markAndAbortRead();
        this.valueField.setValue(newVal);
        if (this.maxValue <= 0xff) {
            dSS.util.setValueOnDevice(this.dsid, this.percentToValue(newVal));
        } else {
            dSS.util.setOutputValueOnDevice(this.dsid, 0x02,
                                            this.percentToValue(newVal));
        }
    },
    handleBladeAngleChanged: function(field, newVal, oldVal) {
        this.markAndAbortReadAngle();
        if (newVal !== this.previousBladeAngle) {
            dSS.util.genericRequest({
                    url: '/json/device/setConfig',
                    scope: this,
                    params:
                    {
                        'class': 64,
                        index: 0x04,
                        dsid: this.dsid,
                        value: Math.round(newVal / 100 * 0xff)
                    },
                    failureMsg: dSS._("Could not set device blade angle")
                });
            this.previousBladeAngle = newVal;
            this.dirtyAngle = false;
        }
    },
    updateSlider: function(field, newVal, oldVal) {
        this.markAndAbortRead();
        var ov = this.valueSlider.getValue();
        if ((newVal !== oldVal) && (newVal !== ov)) {
            this.valueSlider.setValue(newVal);
            if (this.maxValue <= 0xff) {
                dSS.util.setValueOnDevice(this.dsid,
                                          this.percentToValue(newVal));
            } else {
                dSS.util.setOutputValueOnDevice(this.dsid, 0x02,
                                                this.percentToValue(newVal));
            }
        }
    },
    handleMax: function() {
        this.markAndAbortRead();
        dSS.util.callScene(this.dsid, dSS.config.scenes.MAX);
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
    },
    handleMin: function() {
        this.markAndAbortRead();
        dSS.util.callScene(this.dsid, dSS.config.scenes.MIN);
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
    },
    handleStop: function() {
        this.markAndAbortRead();
        dSS.util.callScene(this.dsid, dSS.config.scenes.STOP);
        if (this.matrix.shadeposition === true) {
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dsid ]);
        }
    },
    handleInc: function() {
        this.markAndAbortRead();

        if (this.matrix.shadeposition === true) {
            dSS.util.callScene(this.dsid, dSS.config.scenes.INC);
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dsid ]);
        } else {
            var v = this.valueSlider.getValue() + 5;
            if (v > 100) {
                v = 100;
            }

            if (v != this.valueSlider.getValue()) {
                this.valueSlider.setValue(v);
                this.handleSliderChanged(this.valueSlider, v);
            }
        }
    },
    handleDec: function() {
        this.markAndAbortRead();

        if (this.matrix.shadeposition === true) {
            dSS.util.callScene(this.dsid, dSS.config.scenes.DEC);
            this.dirtyValue = false;
            this.readDeviceValue.defer(500, this, [ this.dsid ]);
        } else {
            var v = this.valueSlider.getValue() - 5;
            if (v < 0) {
                v = 0;
            }

            if (v != this.valueSlider.getValue()) {
                this.valueSlider.setValue(v);
                this.handleSliderChanged(this.valueSlider, v);
            }
        }
    },
    setData: function(dsid, group, present, matrix) {
        this.killRequest();
        this.dsid = dsid;
        this.matrix = matrix;
        if ((matrix.shadeposition == true) && (matrix.outvalue8 === true)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Unsupported configuration"));
            present = false;
        }

        if (matrix.shadeposition === true) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (matrix.shadeposition === true) {
            this.valueLabel.setText(dSS._("Position") + ':');
        } else {
            this.valueLabel.setText(dSS._("Value") + ':');
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);

        if ((group === dSS.config.groups.SHADOW) ||
            (group === dSS.config.groups.ACCESS) ||
            (group === dSS.config.groups.SECURITY)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }
        this.doLayout();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.dirtyValue = false;
        this.dirtyAngle = false;
        if ((present === true) && (matrix.outvalue8 || matrix.shadeposition)) {
            this.enable();
            this.readDeviceValue.defer(0, this, [this.dsid]);
        } else {
            this.disable();
        }
    },
    killRequest: function() {
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
        }
        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }
    },
    killRequestValue: function() {
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
            this.readBladeAngleValue(this.dsid);
        }
    },
    killRequestAngle: function() {
        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }
    },
    readDeviceValue: function(reqdsid)
    {
        this.killRequest();

        if (reqdsid !== this.dsid) {
            return;
        }

        this.fireEvent('loading', true);

        var request = 'getConfigWord';
        var offset = 0x02;

        if (this.maxValue <= 0xff) {
            request = 'getConfig';
            offset = 0;
        }

        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsid: reqdsid
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if ((this.closed === true) ||
                    (this.dirtyValue === true)  ||
                    (reqdsid !== this.dsid)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var p = this.valueToPercent(jsonData.result.value);
                        this.valueSlider.setValue(p);
                        this.valueField.setValue(p);
                    }

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(this.dsid);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(this.dsid);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            }
        });
    },
    markAndAbortRead: function() {
        this.dirtyValue = true;
        this.fireEvent('loading', false);
        this.killRequestValue();
    },
    markAndAbortReadAngle: function() {
        this.dirtyAngle = true;
        this.fireEvent('loading', false);
        this.killRequestAngle();
    },
    readBladeAngleValue: function(reqdsid) {
        this.killRequestValue();

        if (reqdsid !== this.dsid) {
            return;
        }

        this.fireEvent('loading', true);
        var request = 'getConfig';
        var offset = 0x04;

        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsid: reqdsid
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdsid !== this.dsid)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        this.previousBladeAngle = Math.round(jsonData.result.value * 100 / 0xff)
                        this.bladeAngleField.setValue(this.previousBladeAngle);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                }
            }
        });
    }
});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);

// parameters:
//  maskElement
//  device record from the device store
dSS.devices.DeviceSettingsWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        if ((config.devices === undefined) || (config.index === undefined) ||
            (config.devices.length === 0)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No information for device"));
            return;
        }

        if (config.rowSel !== undefined) {
            this.rowSel = config.rowSel;
        }

        var disableLeft = true;
        var disableRight = true;

        this.devices = config.devices;
        this.index = config.index;

        var device = this.devices[this.index];

        if (this.devices.length > 1) {
            if (this.index > 0) {
                disableLeft = false;
            }
            if (this.index < this.devices.length - 1) {
                disableRight = false;
            }
        }

        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(devCls[0], devInfo[0],
                                                       devInfo[1]);
        } catch (error) {
            matrix = error;
        }

        this.left = new Ext.Button({
            iconCls: 'buttonLeftIcon',
            tooltip: dSS._("Select previous device"),
            width: 50,
            disabled: disableLeft,
            handler: this.previousDevice.createDelegate(this)
        });

        this.right = new Ext.Button({
            iconCls: 'buttonRightIcon',
            tooltip: dSS._("Select next device"),
            width: 50,
            disabled: disableRight,
            handler: this.nextDevice.createDelegate(this)
        });

        this.cutWidth = 286;

        this.titleIcon = dSS.util.getDeviceIconCSS(devInfo[0], devCls[0], true);

        var enabled = ((device.get('isPresent') === true) &&
                       (matrix.outvalue8 || matrix.shadeposition));

        Ext.apply(this, {
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            padding: 10,
            width: 320,
            iconCls: enabled ? 'loading-indicator-small' : this.titleIcon,
            resizable: false,
            title: dSS._("Device settings"),
            items:
            [
                {
                    xtype: 'panel',
                    ref: 'info',
                    border: false,
                    items:
                    [
                        dSS.util.CreateDefaultDeviceInfoPanel(
                                                            device.get('name'),
                                                            device.get('dsid'),
                                                            this.cutWidth)
                    ]
                },
                {
                    xtype: 'dssdevsettingspanel',
                    ref: 'settings',
                    dsid: device.get('dsid'),
                    group: devCls[0],
                    present: device.get('isPresent'),
                    matrix: matrix
                }
            ],
            buttonAlign: 'left',
            buttons:
            [
                this.left,
                this.right,
                new Ext.Toolbar.Fill(),
                {
                    xtype: 'button',
                    text: dSS._("Close"),
                    scope: this,
                    handler: function() {
                        this.maskElement.unmask();
                        this.close();
                    }
                }
            ]
        });

        dSS.devices.DeviceSettingsWindow.superclass.constructor.apply(this,
                                                                     arguments);

        this.maskElement.mask();

        this.on("beforedestroy", function(component) {
            this.maskElement.unmask();
        },
        this);

        this.settings.on('loading', function(l) {
            if (l === true) {
                this.setIconClass('loading-indicator-small');
            } else {
                this.setIconClass(this.titleIcon);
            }
        },
        this);

    },
    updateView: function() {
        var device = this.devices[this.index];
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(devCls[0], devInfo[0],
                                                       devInfo[1]);
        } catch (error) {
            matrix = error;
        }

        this.titleIcon = dSS.util.getDeviceIconCSS(devInfo[0], devCls[0], true);
        this.setIconClass(this.titleIcon);

        this.info.removeAll(true);
        this.info.add(
                dSS.util.CreateDefaultDeviceInfoPanel(device.get('name'),
                                                      device.get('dsid'),
                                                      this.cutWidth));
        this.info.doLayout();
        this.settings.setData(device.get('dsid'), devCls[0],
                              device.get('isPresent'), matrix);
        if (this.rowSel !== undefined) {
            this.rowSel.selectRow(this.index, false, false);
        }
    },
    previousDevice: function() {
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.left.disable();
            }

            if ((this.devices.length > 1) &&
                (this.index < (this.devices.length - 1))) {
                this.right.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.right.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.left.enable();
            }
        }
        this.updateView();
    }
});

